@@ -373,6 +373,8 @@ gconf_peditor_new (gchar                 *key,
 	gconf_entry = gconf_client_get_entry (client, GCONF_PROPERTY_EDITOR (obj)->p->key, NULL, TRUE, NULL);
 	GCONF_PROPERTY_EDITOR (obj)->p->callback (client, 0, gconf_entry, obj);
 	GCONF_PROPERTY_EDITOR (obj)->p->inited = TRUE;
+	gconf_entry_free (gconf_entry);
+	g_object_unref (G_OBJECT (client));
 
 	return obj;
 }
@@ -463,6 +465,102 @@ gconf_peditor_new_boolean (GConfChangeSet *changeset,
 	return peditor;
 }
 
+static void
+peditor_integer_value_changed (GConfClient         *client,
+			       guint                cnxn_id,
+			       GConfEntry          *entry,
+			       GConfPropertyEditor *peditor) 
+{
+	GConfValue *value, *value_wid;
+	const char *entry_current_text;
+	int         entry_current_integer;
+
+	if (peditor->p->changeset != NULL)
+		gconf_change_set_remove (peditor->p->changeset, peditor->p->key);
+
+	value = gconf_entry_get_value (entry);
+
+	if (value != NULL) {
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+		entry_current_text = gtk_entry_get_text (GTK_ENTRY (peditor->p->ui_control));
+		entry_current_integer = strtol (entry_current_text, NULL, 10);
+		if (entry_current_integer != gconf_value_get_int (value)) {
+			char *buf = g_strdup_printf ("%d", gconf_value_get_int (value_wid));
+			gtk_entry_set_text (GTK_ENTRY (peditor->p->ui_control), buf);
+			g_free (buf);
+		}
+		gconf_value_free (value_wid);
+	}
+}
+
+static void
+peditor_integer_widget_changed (GConfPropertyEditor *peditor,
+				GtkEntry            *entry)
+{
+	GConfValue *value, *value_wid;
+	  
+	if (!peditor->p->inited) return;
+
+	value_wid = gconf_value_new (GCONF_VALUE_INT);
+
+	gconf_value_set_int (value_wid, strtol (gtk_entry_get_text (entry), NULL, 10));
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
+
+	peditor_set_gconf_value (peditor, peditor->p->key, value);
+
+	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
+	gconf_value_free (value_wid);
+	gconf_value_free (value);
+}
+
+static GObject *
+gconf_peditor_new_integer_valist (GConfChangeSet *changeset,
+				  gchar          *key,
+				  GtkWidget      *entry,
+				  gchar          *first_property_name,
+				  va_list         var_args)
+{
+	GObject *peditor;
+
+	peditor = gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_integer_value_changed,
+		 changeset,
+		 G_OBJECT (entry),
+		 first_property_name,
+		 var_args, NULL);
+
+	g_signal_connect_swapped (G_OBJECT (entry), "changed",
+				  (GCallback) peditor_integer_widget_changed, peditor);
+
+	return peditor;
+}
+
+GObject *
+gconf_peditor_new_integer (GConfChangeSet *changeset,
+			   gchar          *key,
+			   GtkWidget      *entry,
+			   gchar          *first_property_name,
+			   ...)
+{
+	GObject *peditor;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_ENTRY (entry), NULL);
+
+	va_start (var_args, first_property_name);
+
+	peditor = gconf_peditor_new_integer_valist
+		(changeset, key, entry,
+		 first_property_name, var_args);
+
+	va_end (var_args);
+
+	return peditor;
+}
+
 static void
 peditor_string_value_changed (GConfClient         *client,
 			      guint                cnxn_id,
@@ -663,28 +761,23 @@ gconf_peditor_new_color (GConfChangeSet *changeset,
 }
 
 static int
-peditor_enum_int_from_string (GType type, const gchar *str, gboolean *use_nick)
+peditor_enum_int_from_string (GType type, const gchar *str, gboolean use_nick)
 {
 	GEnumClass *klass;
 	GEnumValue *val;
 	int ret = -1;
 
-	if (use_nick)
-		*use_nick = FALSE;
-	
 	klass = g_type_class_ref (type);
-	val = g_enum_get_value_by_name (klass, str);
-	if (!val)
-	{
+	if (use_nick)
 		val = g_enum_get_value_by_nick (klass, str);
-		if (use_nick)
-			*use_nick = TRUE;
-	}
-	if (val)
-		ret = val->value;
-
+	else
+		val = g_enum_get_value_by_name (klass, str);
+	
 	g_type_class_unref (klass);
 
+	if (val)
+		ret = val->value;
+	
 	return ret;
 }
 
@@ -725,7 +818,7 @@ peditor_enum_conv_to_widget (GConfPropertyEditor *peditor,
 	
 	index = peditor_enum_int_from_string (data->enum_type,
 		    		    	      gconf_value_get_string (value),
-					      &data->use_nick);
+					      data->use_nick);
 	
 	gconf_value_set_int (ret, index);
 
@@ -826,6 +919,7 @@ gconf_peditor_new_select_menu_with_enum	(GConfChangeSet *changeset,
 					 gchar 	        *key,
 					 GtkWidget      *option_menu,
 					 GType          enum_type,
+					 gboolean	use_nick,
 					 gchar          *first_property_name,
 					 ...)
 {
@@ -840,6 +934,7 @@ gconf_peditor_new_select_menu_with_enum	(GConfChangeSet *changeset,
 
 	data = g_new0 (GConfPropertyEditorEnumData, 1);
 	data->enum_type = enum_type;
+	data->use_nick = use_nick;
 
 	va_start (var_args, first_property_name);
 
@@ -1041,7 +1136,7 @@ guard_get_bool (GConfPropertyEditor *peditor, const GConfValue *value)
 	else
 	{
 		GConfPropertyEditorEnumData *data = peditor->p->data;
-		int index = peditor_enum_int_from_string (data->enum_type, gconf_value_get_string (value), &data->use_nick);
+		int index = peditor_enum_int_from_string (data->enum_type, gconf_value_get_string (value), data->use_nick);
 		return (index != data->enum_val_false);
 	}
 }
@@ -1082,7 +1177,7 @@ gconf_peditor_widget_set_guard (GConfPropertyEditor *peditor,
 }
 
 GConfValue *
-gconf_value_int_to_float (const GConfValue *value)
+gconf_value_int_to_float (GConfPropertyEditor *ignored, const GConfValue *value)
 {
 	GConfValue *new_value;
 
@@ -1092,7 +1187,7 @@ gconf_value_int_to_float (const GConfValue *value)
 }
 
 GConfValue *
-gconf_value_float_to_int (const GConfValue *value)
+gconf_value_float_to_int (GConfPropertyEditor *ignored, const GConfValue *value)
 {
 	GConfValue *new_value;
 
@@ -1285,7 +1380,7 @@ peditor_enum_toggle_conv_to_widget (GConfPropertyEditor *peditor,
 	
 	index = peditor_enum_int_from_string (data->enum_type,
 		    		    	      gconf_value_get_string (value),
-					      &data->use_nick);
+					      data->use_nick);
 	gconf_value_set_bool (ret, (index != data->enum_val_false));
 
 	return ret;
@@ -1323,6 +1418,7 @@ gconf_peditor_new_enum_toggle  (GConfChangeSet 	 *changeset,
 				GType			 enum_type,
 				GConfPEditorGetValueFn   val_true_fn,
 				guint			 val_false,
+				gboolean		 use_nick,
 				gpointer		 data,
 				gchar 			 *first_property_name,
 				...)
@@ -1340,6 +1436,7 @@ gconf_peditor_new_enum_toggle  (GConfChangeSet 	 *changeset,
 	enum_data->enum_val_true_fn = val_true_fn;
 	enum_data->enum_val_true_fn_data = data;
 	enum_data->enum_val_false = val_false;
+	enum_data->use_nick = use_nick;
 
 	va_start (var_args, first_property_name);
 
@@ -1379,6 +1476,11 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 	gchar *message = NULL;
 	GList *l;
 	
+	/* NULL is not valid, however "" is, but not an error (it's
+	 * the default) */
+	g_return_val_if_fail (filename != NULL, FALSE);
+
+
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
 	{
 		message = g_strdup_printf (_("Couldn't find the file '%s'.\n\nPlease make "
@@ -1396,25 +1498,6 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 					   filename);
 	}
 
-	if (message)
-	{
-		GtkWidget *box;
-
-		box = gtk_message_dialog_new (NULL,
-					      GTK_DIALOG_MODAL,
-					      GTK_MESSAGE_ERROR,
-					      GTK_BUTTONS_OK,
-					      message);
-		gtk_dialog_run (GTK_DIALOG (box));
-		gtk_widget_destroy (box);
-		g_free (message);
-
-		return FALSE;
-	}
-
-	scaled = preview_file_selection_intelligent_scale (pixbuf,
-							   scale);
-
 	if (GTK_IS_IMAGE (GTK_BIN (peditor->p->ui_control)->child))
 		image = GTK_IMAGE (GTK_BIN (peditor->p->ui_control)->child);
 	else
@@ -1423,7 +1506,7 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 		{	
 			if (GTK_IS_IMAGE (l->data))
 				image = GTK_IMAGE (l->data);
-			else if (GTK_IS_LABEL (l->data))
+			else if (GTK_IS_LABEL (l->data) && message == NULL)
 			{
 				gchar *base = g_path_get_basename (filename);
 				gtk_label_set_text (GTK_LABEL (l->data), base);
@@ -1431,6 +1514,32 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 			}
 		}
 	}
+
+	if (message)
+	{
+		if (peditor->p->inited) 
+		{
+			GtkWidget *box;
+			
+			box = gtk_message_dialog_new (NULL,
+						      GTK_DIALOG_MODAL,
+						      GTK_MESSAGE_ERROR,
+						      GTK_BUTTONS_OK,
+						      message);
+			gtk_dialog_run (GTK_DIALOG (box));
+			gtk_widget_destroy (box);
+		} else {
+			gtk_image_set_from_stock (image, GTK_STOCK_MISSING_IMAGE,
+						  GTK_ICON_SIZE_BUTTON);
+		}
+		g_free (message);
+
+		return FALSE;
+	}
+
+	scaled = preview_file_selection_intelligent_scale (pixbuf,
+							   scale);
+
 	gtk_image_set_from_pixbuf (image, scaled);
 	g_object_unref (G_OBJECT (pixbuf));
 	g_object_unref (G_OBJECT (scaled));
@@ -1578,6 +1687,7 @@ gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
 					  gchar		 *key,
 					  GSList 	 *radio_group,
 					  GType 	 enum_type,
+					  gboolean	 use_nick,
 					  gchar          *first_property_name,
 					  ...)
 {
@@ -1594,6 +1704,7 @@ gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
 
 	enum_data = g_new0 (GConfPropertyEditorEnumData, 1);
 	enum_data->enum_type = enum_type;
+	enum_data->use_nick = use_nick;
 
 	first_button = GTK_RADIO_BUTTON (radio_group->data);
 
