@@ -31,6 +31,12 @@
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <config.h>
+
+#include <errno.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <unistd.h>
+
 #include "xsettings-manager.h"
 #include "gnome-settings-daemon.h"
 
@@ -46,6 +52,7 @@
 #include "gnome-settings-default-editor.h"
 #include "gnome-settings-keybindings.h"
 #include "gnome-settings-gtk1theme.h"
+#include "gnome-settings-xrdb.h"
 
 #include "GNOME_SettingsDaemon.h"
 
@@ -272,6 +279,7 @@ gnome_settings_daemon_new (void)
   gnome_settings_background_init (client);
   gnome_settings_keybindings_init (client);
   gnome_settings_gtk1_theme_init (client);
+  gnome_settings_xrdb_init (client);
 
   for (list = directories; list; list = list->next)
     {
@@ -320,6 +328,124 @@ gnome_settings_daemon_new (void)
   gnome_settings_background_load (client);
   gnome_settings_keybindings_load (client);
   gnome_settings_gtk1_theme_load (client);
+  gnome_settings_xrdb_load (client);
 
   return G_OBJECT (daemon);
 }
+
+
+/* Helper functions */
+
+/*
+ * Helper function for spawn_with_input() - wait for a child
+ * to exit.
+ */
+gboolean
+wait_for_child (int  pid,
+		int *status)
+{
+  gint ret;
+
+ again:
+  ret = waitpid (pid, status, 0);
+
+  if (ret < 0)
+    {
+      if (errno == EINTR)
+        goto again;
+      else
+        {
+	  g_warning ("Unexpected error in waitpid() (%s)",
+		     g_strerror (errno));
+	  return FALSE;
+        }
+    }
+
+  return TRUE;
+}
+
+
+/*
+ * Helper function for spawn_with_input() - write an entire
+ * string to a fd.
+ */
+static gboolean
+write_all (int         fd,
+	   const char *buf,
+	   gsize       to_write)
+{
+  while (to_write > 0)
+    {
+      gssize count = write (fd, buf, to_write);
+      if (count < 0)
+	{
+	  if (errno != EINTR)
+	    return FALSE;
+	}
+      else
+	{
+	  to_write -= count;
+	  buf += count;
+	}
+    }
+
+  return TRUE;
+}
+
+/**
+ * gnome_settings_daemon-spawn_with_input:
+ * @argv: command line to run
+ * @input: string to write to the child process. 
+ * 
+ * Spawns a child process specified by @argv, writes the text in
+ * @input to it, then waits for the child to exit. Any failures
+ * are output through g_warning(); if you wanted to use this in
+ * cases where errors need to be presented to the user, some
+ * modification would be needed.
+ **/
+void
+gnome_settings_daemon_spawn_with_input (char       **argv,
+					const char  *input)
+{
+  int exit_status;
+  int child_pid;
+  int inpipe;
+  GError *err = NULL;
+  
+  if (!g_spawn_async_with_pipes (NULL /* working directory */, argv, NULL /* envp */,
+				 G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+				 NULL, NULL, /* child setup and data */
+				 &child_pid,
+				 &inpipe, NULL, NULL, /* stdin, stdout, stderr */
+				 &err))
+    {
+      gchar *command = g_strjoinv (" ", argv);
+      g_warning ("Could not execute %s: %s", command, err->message);
+      g_error_free (err);
+      g_free (command);
+
+      return;
+    }
+
+  if (input)
+    {
+      if (!write_all (inpipe, input, strlen (input)))
+	{
+	  gchar *command = g_strjoinv (" ", argv);
+	  g_warning ("Could not write input to %s", command);
+	  g_free (command);
+	}
+
+      close (inpipe);
+    }
+      
+  wait_for_child (child_pid, &exit_status);
+
+  if (!WIFEXITED (exit_status) || WEXITSTATUS (exit_status))
+    {
+      gchar *command = g_strjoinv (" ", argv);
+      g_warning ("Command %s failed", command);
+      g_free (command);
+    }
+
+}
